[binutils-gdb] Enhance the strings program so that it can display multibyte strings.

Nick Clifton nickc@sourceware.org
Fri Nov 9 12:49:00 GMT 2018


https://sourceware.org/git/gitweb.cgi?p=binutils-gdb.git;h=71f5e3f7b624a725ba550a2dd18a413c88ee6497

commit 71f5e3f7b624a725ba550a2dd18a413c88ee6497
Author: Nick Clifton <nickc@redhat.com>
Date:   Fri Nov 9 12:48:23 2018 +0000

    Enhance the strings program so that it can display multibyte strings.
    
    	* strings.c (print_strings): Check for multibyte encodings.
    	* binutils-all/strings-1.bin: New file.  Test binary for string decoding.
    	* testsuite/binutils-all/strings.exp: New file.  Test the strings program.
    	* testsuite/config/default.exp (STRINGS): Define if not provided
    	by the environment.
    	(STRINGSFLAGS): Likewise.

Diff:
---
 binutils/ChangeLog                            |   9 ++++++
 binutils/strings.c                            |  43 ++++++++++++++++++++++++--
 binutils/testsuite/binutils-all/strings-1.bin | Bin 0 -> 31 bytes
 binutils/testsuite/binutils-all/strings.exp   |  41 ++++++++++++++++++++++++
 binutils/testsuite/config/default.exp         |   6 ++++
 5 files changed, 97 insertions(+), 2 deletions(-)

diff --git a/binutils/ChangeLog b/binutils/ChangeLog
index 2b76d8c..77c61aa 100644
--- a/binutils/ChangeLog
+++ b/binutils/ChangeLog
@@ -1,3 +1,12 @@
+2018-11-09  Mathias   <m4tze@exitno.de>
+
+	* strings.c (print_strings): Check for multibyte encodings.
+	* binutils-all/strings-1.bin: New file.  Test binary for string decoding.
+	* testsuite/binutils-all/strings.exp: New file.  Test the strings program.
+	* testsuite/config/default.exp (STRINGS): Define if not provided
+	by the environment.
+	(STRINGSFLAGS): Likewise.
+
 2018-11-07  Masatake Yamato  <yamato@redhat.com>
 
 	* objdump.c (long_options): Have the --disassemble option take an
diff --git a/binutils/testsuite/binutils-all/strings-1.bin b/binutils/testsuite/binutils-all/strings-1.bin
new file mode 100755
index 0000000..569be85
Binary files /dev/null and b/binutils/testsuite/binutils-all/strings-1.bin differ
diff --git a/binutils/testsuite/binutils-all/strings.exp b/binutils/testsuite/binutils-all/strings.exp
new file mode 100644
index 0000000..c4bbf69
--- /dev/null
+++ b/binutils/testsuite/binutils-all/strings.exp
@@ -0,0 +1,41 @@
+#   Copyright (C) 2018 Free Software Foundation, Inc.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+
+proc test_multibyte {testfile} {
+    global STRINGS
+    global STRINGSFLAGS
+    
+    set testname "strings: decoding little-endian multibyte strings"
+    set got [binutils_run $STRINGS "$STRINGSFLAGS --encoding=l $testfile"]
+
+    set want "String1"
+    if ![regexp $want $got] then {
+	fail "$testname [String1]"
+	return
+    }
+
+    set want "tring2"
+    if ![regexp $want $got] then {
+	fail "$testname [tring2]"
+	return
+    }
+
+    pass $testname
+}
+
+test_multibyte $srcdir/$subdir/strings-1.bin
+
+
diff --git a/binutils/testsuite/config/default.exp b/binutils/testsuite/config/default.exp
index 1bfe72e..b34e45c 100644
--- a/binutils/testsuite/config/default.exp
+++ b/binutils/testsuite/config/default.exp
@@ -63,6 +63,12 @@ if ![info exists OBJCOPYFLAGS] then {
 if ![info exists AR] then {
     set AR [findfile $base_dir/ar]
 }
+if ![info exists STRINGS] then {
+    set STRINGS [findfile $base_dir/strings]
+}
+if ![info exists STRINGSFLAGS] then {
+    set STRINGSFLAGS ""
+}
 if ![info exists STRIP] then {
     set STRIP [findfile $base_dir/strip-new $base_dir/strip-new [transform strip]]
 }



More information about the Binutils-cvs mailing list
diff --git a/binutils/strings.c b/binutils/strings.c
index 74545db..eac7292 100644
--- a/binutils/strings.c
+++ b/binutils/strings.c
@@ -540,9 +540,48 @@ print_strings (const char *filename, FILE *stream, file_ptr address,
 	      free (buf);
 	      return;
 	    }
+
 	  if (! STRING_ISGRAPHIC (c))
-	    /* Found a non-graphic.  Try again starting with next char.  */
-	    goto tryline;
+	    {
+	      /* Found a non-graphic. Try again starting with next char.  */
+	      if (encoding_bytes > 1)
+		{
+		  /* In case of multibyte encodings rewind using magic buffer.  */
+		  if (magiccount == 0)
+		    {
+		      /* If no magic buffer exists: use memory of c.  */
+		      switch (encoding)
+			{
+			default:
+			  break;
+			case 'b':
+			  c = c & 0xff;
+			  magiccount += 1;
+			  break;
+			case 'l':
+			case 'L':
+			  c = c >> 8;
+			  magiccount += (encoding_bytes -1);
+			  break;
+			case 'B':
+			  c = (( c & 0xff0000) >> 16) | ( c & 0xff00)
+			    | (( c & 0xff) << 16);
+			  magiccount += 3;
+			  break;
+			}
+		      magic = (char *) &c;
+		    }
+		  else
+		    {
+		      /* If magic buffer exists: rewind.  */
+		      magic = magic - (encoding_bytes -1);
+		    }
+
+		  address = address - (encoding_bytes -1);
+		}
+
+	      goto tryline;
+	    }
 	  buf[i] = c;
 	}
 